/* FGen_IO_defs-01.C

Timer A1 is used to drive timer C (PPM) in order to enable low frequencies
Timer B is used by HD44780.Lib to generate time delays with DELAY_RESOLUTION resolution

Serial Port C in SPI mode is used to control the AD9834 and MCP4822
Serial Port D in SPI mode is used to generate the single bit ARB signal

Parallel I/O port usage

Port A: Data bus - input and output - NOT External I/O

Port B:
	0,1: reserve for SPI A,B clocks
	2: DAC /CS
	3: LCD_RS: Low = Command,  Data = High
   4: DDS /CS
   5: LCD_ENA: high pulse
	6: Latch Enable for LEDs: pulse high
   7: Output enable for reading switches: high to enable

Port C:
	0: PPM-0 and ARB (SPI TxD)
	3: SPI TxC data for: DDS and DAC

Port D:
	0: Quadrature Decoder - I
	1: Quadrature Decoder - Q
   3: ARB Clock

Port E:
	1: PPM-1
	2: PPM-2
	3: PPM-3
   5: PWM-0
   6: PWM-1
   7: SPI clock C

NOTE: Timer B is used to generate a timing delay required by the LCD.

*/
#define USE_QUAD_ENCODER	// Define IFF you are using a quadrature encoder
									// The LCD hardware-specific library must call the
                           // appropriate functions as well as initialize
                           // the driver.
#ifdef USE_QUAD_ENCODER
	#define	QD1_USEPORTD
	#define	QD_10BIT_OPERATION
	#use		qd.lib
#endif

// Define the control registers and bits
#define	LCD_DATA_PORT	A			// use parallel port A for the data bus
#define	LCD_BITS 		8			// use 8 bit interface

#define	LCD_RS_PORT		B
#define	LCD_RS_BIT		3

#define	LCD_ENA_PORT	B
#define	LCD_ENA_BIT 	5

#define	DOUT_LE_PORT	B
#define	DOUT_LE_BIT		6

#define	DIN_ENA_PORT	B
#define	DIN_ENA_BIT		7

#use HD44780.Lib						// display hardware-specific library

//****** begin definitions required by AD9834.lib, MCP4832 and fser_4.lib

#define	DDS_CS_PORT		B			// chip select register
#define	DDS_CS_BIT		4

#define	DAC_CS_PORT		B			// chip select register
#define	DAC_CS_BIT		2

#define	PB_INIT_STATE		((1<<DDS_CS_BIT)+(1<<DAC_CS_BIT)) // DDS & DAC chip selects high

#define	DDS_AMP_DAC	DAC_A
#define	DDS_OFF_DAC	DAC_B

#define	FUNCTION_GENERATOR FSER_D	// define for Fser.lib
#define	FSERC_ENABLE	// use serial port C for SPI
#define	FSERD_ENABLE	// use serial port D for ARB (SPI)
#use fser_4.lib			// used by AD9834, MCP4822 libraries and the ARB

#define	SPIport			FSER_C
#define	SPI_BAUD			50000L
#define	SPIportW			FSER_C|0x8000
#define	FSERC_SYNC 		SPI_MODE_MASTER
#define	FSERC_CLK		PEDR

#define	ARBport			FSER_D
#define	ARBportW			FSER_D|0x8000
#define	FSERD_SYNC 		SPI_MODE_MASTER
#define	FSERD_CLK		PEDR
#define	ARB_DLR			SDDLR
//****** end definitions required by AD9834.lib, MCP4832 and fser_4.lib

/*
Port A: Data bus - input and output - NOT External I/O

Port B:
	0,1: reserve for SPI A,B clocks
	2: LCD_RS: Low = Command,  Data = High
   3: LCD_ENA: high pulse
	4: Latch Enable for LEDs: pulse high
	5: DAC /CS
   6: DDS /CS
   7: Output enable for reading switches: high to enable

Port C:
	3: SPI TxC data

Port D:
	0: Quadrature Decoder - I
	1: Quadrature Decoder - Q
	2: Output Enable for LEDs: 0 = enable
	3: not used

Port E:
	0: PPM-0
	1: PPM-1
	2: PPM-2
	3: PPM-3
   5: PWM-0
   6: PWM-1
   7: SPI clock C
*/

void InitIO ( void )
{	int i, j;

   WrPortI ( PWL0R, &PWL0RShadow, 0 );	// disable PWM interrupts
	WrPortI ( TAT1R, &TAT1RShadow, 0 );	// TA1 = /1
   WrPortI ( TCCR, &TCCRShadow, 4 );	// disable interrupts, use TA1

// initial set up of parallel ports

//Port A: Data bus - input and output - NOT External I/O
	WrPortI ( PADR, NULL, 0 );				// Port A = 0
	WrPortI ( SPCR, NULL, 0x84 );			// Port A = output

/*			Port B:
	0,1: reserve for SPI A,B clocks
	2: DAC /CS
	3: LCD_RS: Low = Command,  Data = High
   4: DDS /CS
   5: LCD_ENA: high pulse
	6: Latch Enable for LEDs: pulse high
   7: Output enable for reading switches: high to enable
*/
	WrPortI ( PBDR, &PBDRShadow, PB_INIT_STATE );
   WrPortI ( PBDDR, &PBDDRShadow, 0xFF ); // Port B = output

//			Port C:
// 0: PPM-0
	WrPortI ( PCALR, &PCALRShadow, PCALRShadow &~0x03 ); // PC0 = out 0
	WrPortI ( PCALR, &PCALRShadow, PCALRShadow |0x02 ); // PC0 = alt out 2 = PPM0
	BitWrPortI ( PCFR, &PCFRShadow, 1, 0 ); // enable alt function on PC0
	BitWrPortI ( PCDDR, &PCDDRShadow, 1, 0 ); // enable PC0 as output
// 3: SPI TxC data
	WrPortI ( PCALR, &PCALRShadow, PCALRShadow &~0xC0 ); // PC3 = TxC = alt out 0
	BitWrPortI ( PCFR, &PCFRShadow, 1, 3 ); // enable alt function on PC3
	BitWrPortI ( PCDDR, &PCDDRShadow, 1, 3 ); // enable PC3 as output

/*			Port D:
	0: Quadrature Decoder - I
	1: Quadrature Decoder - Q
*/
	WrPortI ( PDDR, &PDDRShadow, 0 );	// Port D = 0
   WrPortI ( PDDDR, &PDDDRShadow, PDDDRShadow&~0x03 ); // PD0,1 = QuadDecoder

//Port E:
//	0: PPM-0
//	1: PPM-1
//	2: PPM-2
//	3: PPM-3
// 5: PWM-0
// 6: PWM-1
// 7: SPI clock C
   PEAHRShadow &= 0x03;						// leave bits for PE4 alone
	PEAHRShadow |= (0x08 | 0x20 | 0xC0); // PE5,6,7 = alt functions
	WrPortI ( PEAHR, &PEAHRShadow, PEAHRShadow );	// bits 7..4
	WrPortI ( PEALR, &PEALRShadow, 0xAA );	// bits 3..0 = alt function 2
	WrPortI ( PEFR,  &PEFRShadow,  PEFRShadow|~0x10 ); // leave bit 4 alone, rest are alt output
	WrPortI ( PEDDR, &PEDDRShadow, PEDDRShadow|~0x10 ); // leave bit 4 alone, rest are output

// Output latch: IC3
   BitWrPortI ( PBDR, &PBDRShadow, 1, DOUT_LE_BIT ); // pulse
   BitWrPortI ( PBDR, &PBDRShadow, 0, DOUT_LE_BIT ); //		LE

	WrPortI ( TCCSR, NULL, 1 );			// enable Timer C clock
   WrPortI ( TAPR, &TAPRShadow, 1 );	// set TAPR to /2

   fSerOpen ( SPIport, SPI_BAUD, 1, FSER_RXC );
   ChParams.DDSPhase[0] = 0;
   ChParams.DDSPhase[1] = 0;
   AD9834init();

} // InitIO

